/***************************************************************************
 *  file                 :  callbacks.c                                    *
 *  copyright            : (C) 2001-2005 by Dirk Nolting                   *
 *  email                : nolting@uni-duesseldorf.de                      *
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <string.h>
#include <stdio.h>
#include <math.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "global.h"
#include "ipc.h"
#include "gui_interaction.h"
#include "pars.h"
#include "ddm.h"

static FormulaType ftype = SUM;
static int edited=0;
static int new_charge=DEFAULT_CHARGE, new_digits=USE_DIGITS; 
static double new_minint=MIN_INT, fastcalc=50, new_fastcalc;
static gchar *last_file=NULL;
static GtkWidget *dlg_options=NULL;
static gboolean fastcalc_active=FALSE, new_show_all=FALSE, show_all=FALSE;

int get_fastcalc()
{
  return rint( fastcalc );
}

gboolean do_fastcalc()
{
  return fastcalc_active;
}

FormulaType get_formulatype()
{
    return ftype;
}

gboolean is_show_all_active()
{
    return show_all;
}

void
on_neu1_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  last_file = NULL;
  clear_results();
  clear_display();
  edited = 0;
}


void
on_speichern1_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    if( !(get_display_data()) )
      {
	display_error_dlg( _("Error while saving:"),
			   _("There are no results to save!"));
	return;
      }

    if( last_file )
	save_result( last_file );
    else
	on_speichern_unter1_activate( menuitem, user_data );
}


void
on_speichern_unter1_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
    GtkWidget *saver;

    if( !(get_display_data()) )
      {
	display_error_dlg( _("Error while saving:"),
			   _("There are no results to save!"));
	return;
      }

    saver = create_filechooser();
    if( last_file)
	gtk_file_chooser_set_filename( GTK_FILE_CHOOSER(saver), last_file);

    gtk_widget_show( saver );

    switch( gtk_dialog_run (GTK_DIALOG (saver)) ) 
    {
	case GTK_RESPONSE_OK:
	    if( last_file )
		g_free( last_file );
	   last_file = gtk_file_chooser_get_filename (
	       GTK_FILE_CHOOSER (saver));

	   save_result( last_file );

	case GTK_RESPONSE_CANCEL:
	
	default:
	    gtk_widget_destroy( saver );
    }

}


void
on_beenden1_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_ausschneiden1_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_kopieren1_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_einf__gen1_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_l__schen1_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_info1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  static GtkWidget *show_info=NULL;
  GdkPixbuf *logo ;
  gchar *license, *authors[]={"Dirk Nolting", "Nick Hoover",NULL};


  license = get_license();
  logo = create_pixbuf( "ccl4.png" );

  if( (!show_info) )
  {
      show_info = gtk_about_dialog_new();
      gtk_about_dialog_set_version( GTK_ABOUT_DIALOG(show_info), VERSION );
      
      gtk_about_dialog_set_copyright( GTK_ABOUT_DIALOG(show_info), 
				      "(c) Dirk Nolting");
      
      gtk_about_dialog_set_comments( GTK_ABOUT_DIALOG(show_info),
				     _("A small tool for mass spectrometry"));
      
      gtk_about_dialog_set_license( GTK_ABOUT_DIALOG(show_info), license );
      
      gtk_about_dialog_set_website( GTK_ABOUT_DIALOG(show_info), 
				    "http://isotopatcalc.sourceforge.net/");
      
      gtk_about_dialog_set_logo( GTK_ABOUT_DIALOG(show_info), logo );
      gtk_about_dialog_set_authors( GTK_ABOUT_DIALOG(show_info), &authors);
  }
      
  gtk_widget_show( show_info );
  free( license );

}


void
on_button_go_released                  (GtkButton       *button,
                                        gpointer         user_data)
{
  G_CONST_RETURN gchar *text;
  GuiInput *infos;

  text = get_formula_input();

  if( !(strlen(text)) )
    {
      display_error_dlg( _("There is no input:"),
			 _("Please enter a formula/sequence first!"));
      return;
    }

  infos = malloc( sizeof( GuiInput ) );

  infos->input_string = strdup( text );
  infos->formulatype = ftype;

  start_calculation( infos );
}


void
on_oeffnen1_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_closebutton1_released               (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget *dialog;

  dialog = gtk_widget_get_toplevel( GTK_WIDGET( button ) );
  gtk_widget_destroy( dialog );
}


void
on_knopf_summenformel_pressed          (GtkButton       *button,
                                        gpointer         user_data)
{
  ftype = SUM;
  //  edited = 1;
}


void
on_knopf_peptidsequenz_pressed         (GtkButton       *button,
                                        gpointer         user_data)
{
  ftype = PEPTIDESEQUENCE;
  //  edited = 0;
}


void
on_sum_formula_menu_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget *dialog=NULL;
  GtkLabel *text;
  PangoAttrList *attrs=NULL;
  gchar *input, *result, *sum_with_markup;

  dialog = create_dlg_sum_formula();
  text = GTK_LABEL( lookup_widget( dialog, "general_information" ) );
  input = (gchar *)get_formula_input();

  if( !(strlen(input)) )
    {
      display_error_dlg( _("There is no sum formula:"),
			 _("Please enter a formula/sequence first!"));
      return;
    }


  if( edited )
    {
      clear_results();
      switch( ftype )
	{
	case PEPTIDESEQUENCE:
	  pars_amino_acid( input );
	  break;
	case SUM:
	  pars_chem_form( input );
	}
      edited = 0;
    }

    sum_with_markup = print_sum();
    pango_parse_markup(sum_with_markup, -1, 0, &attrs, &result, 0, 0);

    gtk_label_set_text( GTK_LABEL( text ), result );
    gtk_label_set_attributes(GTK_LABEL(text), attrs);

    free(sum_with_markup);

    gtk_widget_show( dialog );
}

void
on_info_closebutton_released           (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkWidget *dialog;

  dialog = gtk_widget_get_toplevel( GTK_WIDGET( button ) );
  gtk_widget_destroy( dialog );
}

void
on_formel_input_changed                (GtkEditable     *editable,
                                        gpointer         user_data)
{
  edited = 1;
}


void
on_einstellungen1_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkSpinButton  *chargespin, *digitspin, *intensityspin;
  GtkToggleButton *fast, *show;
  GtkWidget *fast_scale;

  if( dlg_options ) /*don't open dialog twice */
    {
      gtk_window_present( GTK_WINDOW(dlg_options) );
      return;
    }

  dlg_options = create_dlg_options();

  new_charge = get_charge();
  new_digits = get_digits();
  new_minint = get_minint();

  chargespin = (GtkSpinButton *) lookup_widget( dlg_options, "chargespin" );
  digitspin = (GtkSpinButton *) lookup_widget( dlg_options, "digitspin" );
  intensityspin = (GtkSpinButton *) lookup_widget( dlg_options, 
						   "intensityspin" );

  fast_scale = lookup_widget( dlg_options, "fastcalc_scale" );
  gtk_range_set_value( GTK_RANGE( fast_scale ), fastcalc );

  if( fastcalc_active )
    {
      fast  = (GtkToggleButton *) lookup_widget( dlg_options, 
						 "fastcalc_checkbox" );

      gtk_toggle_button_set_active( fast, TRUE );
      gtk_widget_set_sensitive( fast_scale, TRUE );
    }

  gtk_spin_button_set_value( chargespin, new_charge );
  gtk_spin_button_set_value( digitspin, new_digits );
  gtk_spin_button_set_value( intensityspin, new_minint );

  if( show_all )
  {
      show  = (GtkToggleButton *) lookup_widget( dlg_options, 
						 "show_all_button" );
					   
      gtk_toggle_button_set_active( show, TRUE );
      gtk_widget_set_sensitive( GTK_WIDGET(intensityspin), FALSE );
  }

  gtk_widget_show( dlg_options );
}


void
on_canceloptions_released              (GtkButton       *button,
                                        gpointer         user_data)
{
  gtk_widget_destroy( dlg_options );
  dlg_options = NULL;
}

void
on_chargespin_changed                  (GtkSpinButton   *spinbutton,
                                        gpointer         user_data)
{
  new_charge = gtk_spin_button_get_value_as_int( spinbutton );
}


void
on_digitspin_value_changed             (GtkSpinButton   *spinbutton,
                                        gpointer         user_data)
{
  new_digits = gtk_spin_button_get_value_as_int( spinbutton );
}


void
on_intensityspin_value_changed         (GtkSpinButton   *spinbutton,
                                        gpointer         user_data)
{
  new_minint = gtk_spin_button_get_value( spinbutton );
}


void
on_optionsok_released                  (GtkButton       *button,
                                        gpointer         user_data)
{
  set_charge( new_charge );
  set_digits( new_digits );
  set_minint( new_minint );
  fastcalc = new_fastcalc;
  show_all = new_show_all;
  on_canceloptions_released(button, user_data);
  display_result();
}


void
on_fastcalc_checkbox_toggled           (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
  GtkWidget *scale;

  scale = lookup_widget( dlg_options, "fastcalc_scale");

  if( gtk_toggle_button_get_active( togglebutton ) )
    {      
      gtk_widget_set_sensitive( scale, TRUE );
      fastcalc_active = TRUE;
      new_fastcalc = gtk_range_get_value( GTK_RANGE(scale) );
    }
  else
    {
      gtk_widget_set_sensitive( scale, FALSE );
      fastcalc_active = FALSE;
    }
}


void
on_fastcalc_scale_value_changed        (GtkRange        *range,
                                        gpointer         user_data)
{
  new_fastcalc = gtk_range_get_value( range );
}


void
on_show_all_button_toggled             (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
    GtkWidget *spin_int;

    spin_int = lookup_widget( dlg_options, "intensityspin");

    if( gtk_toggle_button_get_active( togglebutton ) )
    {
	new_show_all = TRUE;
	gtk_widget_set_sensitive( spin_int, FALSE );
    }
    else
    {
	new_show_all = FALSE;
	gtk_widget_set_sensitive(spin_int, TRUE );
    }
}


void
on_cancel_calc_button_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
    set_cancel_calculation( TRUE );
}

