/***************************************************************************
 *  file                 :  ddm.c (data_display_management)                *
 *  copyright            : (C) 2001-2005 by Dirk Nolting                   *
 *  email                : nolting@uni-duesseldorf.de                      *
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "global.h"
#include "gui_interaction.h"
#include "ipc.h"
#include "ddm.h"
#include "callbacks.h"

#include <math.h>

static int charge, permutationen=0;
static char *massformat = NULL, *intformat = NULL;
static isotope *display=NULL;
static double sump, maxp;

static void clear_display_data(void);


/**
 *clear previous results
 */
void clear_display_data()
{
    if( display ){
	free_list( display );
	display = NULL;
    }

    if( massformat ){
	free( massformat );
	massformat = NULL;
    }

    if( intformat ){
	free( intformat );
	intformat = NULL;
    }

  permutationen = 0;
  sump = 0;
  maxp = 0;
}

/**
 * returns the data list for display
 */
isotope *get_display_data()
{
  return display;
}


/**
 *returns the maximum probability
 */
double get_maxp()
{
  return maxp;
}


/**
 * returns the sum of the probabilities of all peaks
 */
double get_sump()
{
    return sump;
}


/**
 *returns a format string for mass display
 */
char *get_massformat()
{
  return massformat;
}


/**
 *returns a format string for intensity display
 */
char *get_intformat()
{
  return intformat;
}

/**
 * Generate format string for variable output of floats
 */
char *do_format_string( int digits )
  {
    char *format_string;

    format_string = malloc( 10*sizeof(char) );

    if( digits == SHOW_ALL_INT )
	sprintf( format_string,"%%e" );
    else
	sprintf( format_string,"%%.%if", digits);

    return format_string;
  }


/**
 * returns number of calculated permutations
 */
int get_permutations()
{
  return permutationen;
}


/**
 * create format string with precision adjusted to value
 */
char *do_adapted_format_string( double value )
  {
    char *form;
    double lg;
    int digits;
    
    lg = log10( value );
    digits = ceil( ABS(lg) );

    form = do_format_string( digits );

    return form;
  }


/**
 * duplicates a single peak
 */
isotope *duplicate_peak(const isotope *peak){
  isotope *new;

  new = malloc( sizeof(isotope) );
  new->next = NULL;
  new->previous = NULL;
  new->mass = peak->mass;
  new->p = peak->p;

  return new;
}


/*
 * Just append a new element to an isotope list
 */
isotope *append_peak( isotope *base, isotope *new)
  {
    static isotope *reiter;

    if( !base )
      {
	base = new;
	reiter = new;

	return new;
      }

    reiter->next = new;
    new->previous = reiter;
    reiter = new;

    return base;
  }


/**
 * Prepares the data for output (display, save, print)
 */
void generate_display_data( isotope *results )
{
  int digits;
  double minint;
  isotope *d, *reiter;
  gboolean show_all;

  clear_display_data();

  digits = get_digits();
  minint = get_minint();
  charge = get_charge();
  massformat = do_format_string( digits );

  if( (show_all = is_show_all_active()) )
      intformat = do_format_string( SHOW_ALL_INT );      
  else
      intformat = do_adapted_format_string( minint );
 
  for(d=results;d;d=d->next)
    {
      permutationen++;
      sump+=d->p;
      
      reiter = duplicate_peak( d );
      display = append_peak( display, reiter );
      
      reiter->mass=d->mass / charge;
      reiter->mass=(rint( reiter->mass * pow(10, digits) ) / pow(10, digits) );
    }

  summarize_peaks( display );

  for(d=display; d; d=d->next)
    if(d->p > maxp)
      maxp=d->p;
}
