/***************************************************************************
 *  file                 :  ipc.c                                          *
 *  copyright            : (C) 2001-2005 by Dirk Nolting                   *
 *  email                : nolting@uni-duesseldorf.de                      *
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ipc.h"
#include "pars.h"
#include "element.h"
#include "support.h"
#include "gui_interaction.h"
#include "callbacks.h"
#include "interface.h"

#include <time.h>
#include <signal.h>
#include <math.h>

compound *verbindung=NULL;
isotope *peaks=NULL;
static int permutationen;
static gboolean cancel_calculation;
static GtkProgressBar *progress_bar;

gboolean update_progress(gpointer data)
{
  gtk_progress_bar_pulse( progress_bar );

  return !cancel_calculation;
}


void set_cancel_calculation(gboolean cancel)
{
    cancel_calculation = cancel;
}

void free_list(isotope *target)
{

  if( !(target) )
    return;

  while(target->next)
    {
      target=target->next;
      free(target->previous);
    }
  free(target);
}

void free_compound( compound *target )
{

  if( !(target) )
    return;

  while( target->next )
    {
      target=target->next; 
      free( target->previous );
    }
  free( target );
}


/**
 * Clear old results
 */
void clear_results()
{
  free_list(peaks);
  peaks = NULL;
  
  free_compound( verbindung );
  verbindung = NULL;
  cancel_calculation = FALSE;
}


void cut_peaks(isotope *spectrum)
{
  int dummy=1, fastcalc, peak_zahl;

  fastcalc = get_fastcalc();
  peak_zahl = fastcalc * permutationen / 100;

  while( (spectrum->next) && (dummy<fastcalc) )
    {
      dummy++;
      spectrum=spectrum->next;
    }

  if(spectrum->next)
    {
      free_list(spectrum->next);
      spectrum->next=NULL;
    }
}


void summarize_peaks(isotope *base)
{  
  isotope *dummy,*d2;

  permutationen = 0;
 for(dummy=base;dummy;dummy=dummy->next)
   {
   /* Differenz wegen Rundungsfehlern */
    while( dummy->next && (dummy->next->mass - dummy->mass < 0.00000001) )
      {
	d2=dummy->next;
	dummy->next=d2->next;
	if(dummy->next)
	  dummy->next->previous=dummy;
	dummy->p+=d2->p;
	free(d2);
      }
    permutationen++;
   }
}


/**
 * Add a newly calculated permutation and sort it into the previous results
 */
isotope *add_peak(isotope *base,isotope *peak)
{
 static isotope *reiter;

  if(!(base->mass))
    {
      peak->next=NULL;
      peak->previous=NULL;
      reiter=peak;
      return peak;
    }

  if( peak->mass >= reiter->mass )
    while( (reiter->next) && (reiter->mass < peak->mass))
      reiter=reiter->next;
  else
    {
      while( (reiter->previous) && (reiter->mass > peak->mass) )
        reiter=reiter->previous;
      reiter=reiter->next;
    }

  if( (reiter->mass) >= (peak->mass) )
    {
      peak->next=reiter;
      peak->previous=reiter->previous;
      peak->previous->next=peak;
      reiter->previous=peak;
      return base;
    }
  else
    {
      reiter->next=peak;
      peak->next=NULL;
      peak->previous=reiter;
      return base;
    }
  return 0;
}

int calculate_peaks(){
  compound *c;
  isotope *npeaks,*p,*i,*np1;
  int anzahl, count_for_update=0;

  if(!(peaks=malloc(sizeof(isotope))))
    return 0;
  peaks->mass=0;
  peaks->p=1;
  peaks->previous=NULL;
  peaks->next=NULL;

  for(c=verbindung;c;c=c->next){
    for(anzahl=0;anzahl < c->amount;anzahl++){

      if(!(npeaks=malloc(sizeof(isotope))))
	return 0;
      npeaks->mass=0;

      for(p=peaks;p;p=p->next){
	for(i=c->isotopes;i;i=i->next){

	  if(!(np1=malloc(sizeof(isotope))))
	    return 0;

	  np1->mass=p->mass + i->mass;
	  np1->p=p->p * i->p;
	  if( !(npeaks=add_peak(npeaks,np1)) )
	    return 0;

	  if( (count_for_update++) > 1000 ){
	      count_for_update = 0;
	      while( gtk_events_pending() )
		  gtk_main_iteration();
	  }
	
	  if( cancel_calculation )
	      break;
	}
	if( cancel_calculation )
	    break;
      }
      free_list(peaks);
      peaks=npeaks;
      
      summarize_peaks( peaks );
    }
    if( do_fastcalc () )
	cut_peaks(peaks);
  }
  return 1;
}


void start_calculation(GuiInput *user_input)
{
  GtkWidget *dlg_progressbar;
  guint update_function;

#ifdef HAVE_TIME
  long seconds;
  seconds=time(0);
#endif

  clear_results();

  switch( user_input->formulatype ){
  case SUM:
    pars_chem_form( user_input->input_string );
    break;
  case PEPTIDESEQUENCE:
    pars_amino_acid( user_input->input_string );
    break;
  }

  dlg_progressbar = create_dlg_fortschritt();
  progress_bar = (GtkProgressBar *)lookup_widget( dlg_progressbar, 
						 "progress_calculation");
  gtk_progress_bar_set_pulse_step( progress_bar, 0.2 );
  gtk_progress_bar_set_orientation( progress_bar, GTK_PROGRESS_LEFT_TO_RIGHT);
  gtk_widget_show( dlg_progressbar );
  update_function = g_timeout_add( 100, update_progress, NULL);

  calculate_peaks();

  g_source_remove( update_function );
  gtk_widget_destroy( dlg_progressbar );

  display_result();

  free( user_input->input_string );

#ifdef HAVE_TIME
  seconds=time(0)-seconds;
  printf("Computing time: %li seconds.\n",seconds);
#endif

}
